//---------------------------------------------------------------------------
//  .
// Coded by G3.
// mailto:tgm80@mail.ru

#include <vcl.h>
#pragma hdrstop

#include "main.h"
#include "generate.h"
#include "size.h"
#include "about.h"
#include "palette.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "size"
#pragma link "SHDocVw_OCX"
#pragma link "SHDocVw_OCX"
#pragma resource "*.dfm"
TMainForm *MainForm;

#define HTML_Filename  "tgm.htm"
#define Image_Filename "tgm_bg.bmp"

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{
  pBmp8 = new Graphics::TBitmap();
  pBmp24 = new Graphics::TBitmap();
  pJpeg = new TJPEGImage();
  pGif = new TGIFImage(); //  TGIFImage - Anders Melander
  pJpeg->CompressionQuality = 100;  //   jpeg

  char pStrBuff[256];
  int BuffSize = 256;
  GetTempPath(BuffSize, pStrBuff);
  TempDir = pStrBuff; //   

}

//---------------------------------------------------------------------------
//  
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  randomize();
  X = 100;          //   
  Y = 100;          //   
  Width = 550;      //    
  Height = 400;     //    

  pBmp8->PixelFormat = pf8bit;  // 8   
  pBmp8->Palette = ColorPalette(ColorDialog1->Color);
  SelBG->FilterIndex = 5;

  PageControl1->ActivePage=TabSheet1;
  Memo1->Lines->SaveToFile(TempDir+HTML_Filename);

  GenerateBtnClick(Sender);
}

//---------------------------------------------------------------------------
//   
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  //  
  delete pBmp8;
  delete pBmp24;
  delete pJpeg;
  delete pGif;

  //   
  DeleteFile(TempDir+Image_Filename);
  DeleteFile(TempDir+HTML_Filename);
}

//---------------------------------------------------------------------------
//    
void __fastcall TMainForm::GenerateBtnClick(TObject *Sender)
{
  r1  = (rand()+1)/(float)RAND_MAX;
  r2  = (rand()+1)/(float)RAND_MAX;
  r3  = (rand()+1)/(float)RAND_MAX;
  Generate();
}

//---------------------------------------------------------------------------
//   
void __fastcall TMainForm::ColorBtnClick(TObject *Sender)
{
  if(ColorDialog1->Execute())
  {
    pBmp8->Palette = ColorPalette(ColorDialog1->Color);
    Generate();
  }
}

//---------------------------------------------------------------------------
//  
void __fastcall TMainForm::SaveBtnClick(TObject *Sender)
{
  AnsiString Filename;
  static int NumSaves = 1;
  SavePictureDialog1->FileName = "Texture"+IntToStr(NumSaves);
  if(SavePictureDialog1->Execute())
    if(SaveTexture(SavePictureDialog1,1))
      NumSaves ++;

}

//---------------------------------------------------------------------------
//   
void __fastcall TMainForm::ChangeCanvas(TObject *Sender)
{
  if (!TileBtn->Down)
    PaintBox1->Canvas->FillRect(Rect(0,0,PaintBox1->Width,PaintBox1->Height));

  UpdateCanvas();
}

//---------------------------------------------------------------------------
//  
void __fastcall TMainForm::UpdateCanvas()
{
  TRect   SourceRect, DestRect;

  SourceRect = Rect(0,0,X,Y);

  if (TileBtn->Down)
  {
    for(int i=0; i<PaintBox1->Height/Y+1; i++)
      for(int j=0; j<PaintBox1->Width/X+1; j++)
      {
        DestRect = Rect(j*X, i*Y, (j+1)*X, (i+1)*Y);
        PaintBox1->Canvas->CopyRect(DestRect,pBmp8->Canvas, SourceRect);
      }
  }
  else
  {
    DestRect = Rect((PaintBox1->Width-X)/2, (PaintBox1->Height-Y)/2,
                (PaintBox1->Width-X)/2+X, (PaintBox1->Height-Y)/2+Y);
    PaintBox1->Canvas->CopyRect(DestRect,pBmp8->Canvas, SourceRect);
  }

}

//---------------------------------------------------------------------------
//   
void __fastcall TMainForm::SizeBtnClick(TObject *Sender)
{
  if(SizeForm->ShowModal() == mrOk)
    Generate();
}

//---------------------------------------------------------------------------
//   About
void __fastcall TMainForm::AboutBtnClick(TObject *Sender)
{
  AboutForm->ShowModal();
}

//---------------------------------------------------------------------------
//  
void __fastcall TMainForm::Generate()
{
  pBmp8->Height = Y;
  pBmp8->Width = X;
  CreateTexture(pBmp8, SmoothBtn->Down, r1, r2, r3, ContrastBar->Position/100.0);

  UpdateCanvas();

  //    Test Font Color
  pBmp8->SaveToFile(TempDir+Image_Filename);

  //    Test With Your HTML
  if(SelBG->FileName != "")
    SaveTexture(SelBG,0);

}

//---------------------------------------------------------------------------
//    (TabSheet)
void __fastcall TMainForm::PageControl1Change(TObject *Sender)
{
  if (PageControl1->ActivePage == TabSheet2)
  {
    TVariant Flags = 14;
    TVariant dest = TempDir+HTML_Filename;
    Browser->Navigate2(&dest, &Flags, NULL, NULL, NULL);
  }
  if (PageControl1->ActivePage == TabSheet3)
    TestBrowserReload();
}

//---------------------------------------------------------------------------
//    HTML
void __fastcall TMainForm::ColorFontBtnClick(TObject *Sender)
{
  if(ColorDialog2->Execute())
  {
    int Col =ColorDialog2->Color;
    Col   = ((Col & 0xFF0000)>>16)+(Col & 0x00FF00)+((Col & 0x0000FF)<<16);

    Memo1->Lines->Strings[6] = "text=\"#"+IntToHex(Col,6)+"\"";
    Memo1->Lines->Strings[7] = "link=\"#"+IntToHex(Col,6)+"\"";
    Memo1->Lines->Strings[8] = "vlink=\"#"+IntToHex(Col,6)+"\"";
    Memo1->Lines->SaveToFile(TempDir+HTML_Filename);

    TVariant Flags = 14;
    TVariant dest = TempDir+HTML_Filename;
    Browser->Navigate2(&dest, &Flags, NULL, NULL, NULL);
  }
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateTexture(TObject *Sender)
{
  Generate();
}

//---------------------------------------------------------------------------
//   HTML
void __fastcall TMainForm::SeHTMLBtnClick(TObject *Sender)
{
  if(SelHTML->Execute())
    TestBrowserReload();

}

//---------------------------------------------------------------------------
//    (  )
void __fastcall TMainForm::SelBGBtnClick(TObject *Sender)
{
  if(SelBG->Execute())
  {
    if(SaveTexture(SelBG,1))
      TestBrowserReload();
    else
      SelBG->FileName = "";
  }

}

//---------------------------------------------------------------------------
//  HTML
void __fastcall TMainForm::TestBrowserReload()
{
  if(SelHTML->FileName != "")
  {
    TVariant Flags = 14;
    TVariant dest = SelHTML->FileName;
    TestBrowser->Navigate2(&dest, &Flags, NULL, NULL, NULL);
  }
}

//---------------------------------------------------------------------------
//   About   
void __fastcall TMainForm::FormResize(TObject *Sender)
{
  AboutBtn->Left = Width -  46;
}

//---------------------------------------------------------------------------
//     
bool __fastcall TMainForm::SaveTexture(TSavePictureDialog *SPD, bool CheckOnExists)
{
  AnsiString FileName = SPD->FileName;

  switch(SPD->FilterIndex)
  {
    case 1: //bmp 24 bit
    case 2: //bmp 8 bit
      FileName = ChangeFileExt(SPD->FileName,".bmp");
      break;
    case 3: //jpg
      if(ExtractFileExt(FileName) != ".jpeg")
        FileName = ChangeFileExt(SPD->FileName,".jpg");
      break;
    case 4: //gif
      FileName = ChangeFileExt(SPD->FileName,".gif");
      break;
  };

  if(CheckOnExists)
    if(FileExists(FileName))
      if ( Application->MessageBox("File exists. Overwrite?", "Confirmation", MB_OKCANCEL) != IDOK)
        return false;


  switch(SPD->FilterIndex)
  {
    case 1: //bmp 24 bit
      pBmp24->Assign(pBmp8);
      pBmp24->PixelFormat = pf24bit;
      pBmp24->SaveToFile(FileName);
      break;
    case 2: //bmp 8 bit
      pBmp8->SaveToFile(FileName);
      break;
    case 3: //jpg
      pJpeg->Assign(pBmp8);
      pJpeg->SaveToFile(FileName);
      break;
    case 4: //gif
      pGif->Assign(pBmp8);
      pGif->SaveToFile(FileName);
      break;
    default:
      if(ExtractFileExt(FileName) == ".gif")
      {
        pGif->Assign(pBmp8);
        pGif->SaveToFile(FileName);
      }
      else if(ExtractFileExt(FileName) == ".jpg" || ExtractFileExt(FileName) == ".jpeg")
      {
        pJpeg->Assign(pBmp8);
        pJpeg->SaveToFile(FileName);
      }
      else
      {
        pBmp24->Assign(pBmp8);
        pBmp24->PixelFormat = pf24bit;
        pBmp24->SaveToFile(FileName);
      }
      break;
  };
  return true;
}


